/*
Copyright 2020 Appboxo pte. ltd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package com.appboxo.appboxo_sdk

import android.app.Activity
import androidx.annotation.NonNull
import com.appboxo.js.params.CustomEvent
import com.appboxo.sdk.Appboxo
import com.appboxo.sdk.Config
import com.appboxo.sdk.MiniApp
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

class AppboxoSdkPlugin(private val activity: Activity) : MethodCallHandler {

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "setConfig" -> {
                val clientId = call.argument<String>("client_id")!!
                Appboxo.setConfig(Config.Builder()
                        .setClientId(clientId)
                        .build())
            }
            "openMiniApp" -> {
                val appId = call.argument<String>("app_id")!!
                val payload = call.argument<String>("payload") ?: ""
                val additionalUserFields = call.argument<Map<String, String>>("additional_user_fields")
                val data = call.argument<Map<String, Any>>("data")
                Appboxo.getMiniApp(appId, payload, additionalUserFields, data)
                        .setCustomEventListener { _, miniApp, customEvent ->
                            customEventsStreamHandler.events?.success(
                                    mapOf("appId" to miniApp.appId,
                                            "requestId" to customEvent.requestId,
                                            "type" to customEvent.type,
                                            "errorType" to customEvent.errorType,
                                            "payload" to customEvent.payload))
                        }
                        .setLifecycleListener(object : MiniApp.LifecycleListener {
                            override fun onClose(miniApp: MiniApp) {
                                lifecycleEventsStreamHandler.events?.success(
                                        mapOf(
                                                "appId" to miniApp.appId,
                                                "lifecycle" to "onClose"
                                        )
                                )
                            }

                            override fun onError(miniApp: MiniApp, message: String) {
                                lifecycleEventsStreamHandler.events?.success(
                                        mapOf(
                                                "appId" to miniApp.appId,
                                                "lifecycle" to "onError",
                                                "error" to message
                                        )
                                )
                            }

                            override fun onLaunch(miniApp: MiniApp) {
                                lifecycleEventsStreamHandler.events?.success(
                                        mapOf(
                                                "appId" to miniApp.appId,
                                                "lifecycle" to "onLaunch"
                                        )
                                )
                            }

                            override fun onPause(miniApp: MiniApp) {
                                lifecycleEventsStreamHandler.events?.success(
                                        mapOf(
                                                "appId" to miniApp.appId,
                                                "lifecycle" to "onPause"
                                        )
                                )
                            }

                            override fun onResume(miniApp: MiniApp) {
                                lifecycleEventsStreamHandler.events?.success(
                                        mapOf(
                                                "appId" to miniApp.appId,
                                                "lifecycle" to "onResume"
                                        )
                                )
                            }
                        })
                        .open(activity)
            }
            "hideAllMiniApps" -> {
                Appboxo.hideAllMiniApps()
            }

            "sendEvent" -> {
                try {
                    val map = call.argument<Map<String, *>>("custom_event")!!
                    val appId = map["appId"] as String

                    @Suppress("UNCHECKED_CAST")
                    val customEvent = CustomEvent(requestId = map["requestId"] as? String,
                            type = map["type"] as String,
                            errorType = map["errorType"] as? String,
                            payload = map["payload"] as Map<String, Any>
                    )
                    Appboxo.getMiniApp(appId)?.sendEvent(customEvent)
                } catch (e: Exception) {
                    print(e)
                }
            }
        }
    }

    companion object {
        private val customEventsStreamHandler = CustomEventsStreamHandler()
        private val lifecycleEventsStreamHandler = LifecycleEventsStreamHandler()

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            Appboxo.init(registrar.activity().application)
            val channel = MethodChannel(registrar.messenger(), "appboxo_sdk")
            val customEventChannel = EventChannel(registrar.messenger(), "appboxo_sdk_custom_events")
            val lifecycleChannel = EventChannel(registrar.messenger(), "appboxo_sdk_lifecycle")
            val plugin = AppboxoSdkPlugin(registrar.activity())
            channel.setMethodCallHandler(plugin)
            lifecycleChannel.setStreamHandler(lifecycleEventsStreamHandler)
            customEventChannel.setStreamHandler(customEventsStreamHandler)
        }
    }
}
