/*
 Copyright 2020 Appboxo pte. ltd.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 
 http://www.apache.org/licenses/LICENSE-2.0
 
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

import Flutter
import UIKit
import AppBoxoSDK


public class SwiftAppboxoSdkPlugin: NSObject, FlutterPlugin {
    
    private let customEventsStreamHandler = CustomEventsStreamHandler()
    private let lifecycleEventsStreamHandler = LifecycleEventsStreamHandler()
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = SwiftAppboxoSdkPlugin()
        
        let channel = FlutterMethodChannel(name: "appboxo_sdk", binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(instance, channel: channel)
        
        let customEventChannel = FlutterEventChannel(name: "appboxo_sdk_custom_events", binaryMessenger: registrar.messenger())
        let lifecycleChannel = FlutterEventChannel(name: "appboxo_sdk_lifecycle", binaryMessenger: registrar.messenger())
        
        customEventChannel.setStreamHandler(instance.customEventsStreamHandler)
        lifecycleChannel.setStreamHandler(instance.lifecycleEventsStreamHandler)
    }
    
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        
        switch call.method {
        case "setConfig":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let clientId = arguments["client_id"] as? String ?? ""
            AppBoxo.shared.setConfig(config: Config(clientId: clientId))
        case "openMiniApp":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let appId = arguments["app_id"] as? String ?? ""
            let payload = arguments["payload"] as? String ?? ""
            let additionalUserFields = arguments["additional_user_fields"] as? [String: Any] ?? nil
            let data = arguments["data"] as? [String: Any] ?? nil
            let miniApp = AppBoxo.shared.getMiniApp(appId: appId, authPayload: payload, additionalUserFields: additionalUserFields, data: data)
            miniApp.delegate = self
            miniApp.open(viewController: UIApplication.shared.delegate!.window!!.rootViewController!)
            break
        case "sendEvent":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let map = arguments["custom_event"] as? [String: Any] ?? [:]
            let appId = map["appId"] as? String ?? ""
            
            var params = [
                "request_id" : map["requestId"] ?? 0,
                "type" : map["type"] ?? "",
                "payload" : map["payload"] ?? [String : Any]()
                ] as [String: Any]
            
            if let errorType = map["errorType"] as? String {
                params["error_type"] = errorType
            }
            
            AppBoxo.shared.getMiniApp(appId: appId)?.sendEvent(params: params)
            break
        case "hideAllMiniApps":
            AppBoxo.shared.hideAllMiniApps()
            break
        default:
            break
        }
    }
}

extension SwiftAppboxoSdkPlugin : MiniAppDelegate {
    public func didReceiveCustomEvent(miniApp: MiniApp, params: [String : Any]) {
        var dict = [
            "appId" : miniApp.appId,
            "requestId" : "\(params["request_id"] ?? 0)",
            "type" : params["type"] ?? "",
            "payload" : params["payload"] ?? [String : Any]()
            ] as [String: Any]
        
        if let errorType = params["errorType"] as? String {
            dict["error_type"] = errorType
        }
        
        customEventsStreamHandler.events?(dict)
    }
    
    public func onLaunch(miniApp: MiniApp) {
        let dict = [
            "appId" : miniApp.appId,
            "lifecycle" : "onLaunch"
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
    
    public func onResume(miniApp: MiniApp) {
        let dict = [
            "appId" : miniApp.appId,
            "lifecycle" : "onResume"
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
    
    public func onPause(miniApp: MiniApp) {
        let dict = [
            "appId" : miniApp.appId,
            "lifecycle" : "onPause"
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
    
    public func onClose(miniApp: MiniApp) {
        let dict = [
            "appId" : miniApp.appId,
            "lifecycle" : "onClose"
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
    
    public func onError(miniApp: MiniApp, message: String) {
        let dict = [
            "appId" : miniApp.appId,
            "lifecycle" : "onError",
            "error" : message
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
}
